@@ -53,14 +53,22 @@ Supported, but currently not set.
 - Webservice
 """
 
-from typing import Optional, Sequence
+import os
+from typing import Optional, Sequence, TypeVar, Generic, List
 from dataclasses import dataclass
 from email.utils import parseaddr
+from urllib.parse import urlparse
+
+
+try:
+    from typing import TypedDict  # type: ignore
+except ImportError:
+    from typing_extensions import TypedDict  # type: ignore
 
 
 SUPPORTED_CERTAINTIES = ["certain", "confident", "likely", "possible", None]
 
-version_string = "0.1.25"
+version_string = "0.1.35"
 
 USER_AGENT = "upstream-ontologist/" + version_string
 # Too aggressive?
@@ -74,9 +82,20 @@ class Person:
     email: Optional[str] = None
     url: Optional[str] = None
 
+    def __init__(self, name, email=None, url=None):
+        self.name = name
+        self.email = email
+        if url and url.startswith('mailto:'):
+            self.email = url[len('mailto:'):]
+            self.url = None
+        else:
+            self.url = url
+
     @classmethod
     def from_string(cls, text):
         text = text.replace(' at ', '@')
+        text = text.replace(' -at- ', '@')
+        text = text.replace(' -dot- ', '.')
         text = text.replace('[AT]', '@')
         if '(' in text and text.endswith(')'):
             (p1, p2) = text[:-1].split('(', 1)
@@ -97,16 +116,25 @@ class Person:
 
     def __str__(self):
         if self.email:
-            return '%s <%s>' % (self.name, self.email)
+            return '{} <{}>'.format(self.name, self.email)
         return self.name
 
 
-class UpstreamDatum(object):
+T = TypeVar('T')
+
+
+class UpstreamDatum(Generic[T]):
     """A single piece of upstream metadata."""
 
     __slots__ = ["field", "value", "certainty", "origin"]
 
-    def __init__(self, field, value, certainty=None, origin=None):
+    field: str
+    value: T
+    certainty: Optional[str]
+    origin: Optional[str]
+
+    def __init__(self, field: str, value: T, certainty: Optional[str] = None,
+                 origin: Optional[str] = None) -> None:
         self.field = field
         if value is None:
             raise ValueError(field)
@@ -126,10 +154,10 @@ class UpstreamDatum(object):
         )
 
     def __str__(self):
-        return "%s: %s" % (self.field, self.value)
+        return "{}: {}".format(self.field, self.value)
 
     def __repr__(self):
-        return "%s(%r, %r, %r, %r)" % (
+        return "{}({!r}, {!r}, {!r}, {!r})".format(
             type(self).__name__,
             self.field,
             self.value,
@@ -138,7 +166,20 @@ class UpstreamDatum(object):
         )
 
 
-class UpstreamPackage(object):
+UpstreamMetadata = TypedDict('UpstreamMetadata', {
+    'Name': UpstreamDatum[str],
+    'Contact': UpstreamDatum[str],
+    'Repository': UpstreamDatum[str],
+    'Repository-Browse': UpstreamDatum[str],
+    'X-Summary': UpstreamDatum[str],
+    'Bug-Database': UpstreamDatum[str],
+    'Bug-Submit': UpstreamDatum[str],
+    'Homepage': UpstreamDatum[str],
+    'Screenshots': UpstreamDatum[List[str]],
+}, total=False)
+
+
+class UpstreamPackage:
     def __init__(self, family, name):
         self.family = family
         self.name = name
@@ -169,12 +210,12 @@ def confidence_to_certainty(confidence: Optional[int]) -> str:
         return "unknown"
     try:
         return SUPPORTED_CERTAINTIES[confidence] or "unknown"
-    except IndexError:
-        raise ValueError(confidence)
+    except IndexError as e:
+        raise ValueError(confidence) from e
 
 
 def certainty_sufficient(
-    actual_certainty: str, minimum_certainty: Optional[str]
+    actual_certainty: Optional[str], minimum_certainty: Optional[str]
 ) -> bool:
     """Check if the actual certainty is sufficient.
 
@@ -198,7 +239,17 @@ def certainty_sufficient(
 def _load_json_url(http_url: str, timeout: int = DEFAULT_URLLIB_TIMEOUT):
     from urllib.request import urlopen, Request
     import json
-    headers = {'User-Agent': USER_AGENT, 'Accept': 'application/json'}
+    headers = {
+        'User-Agent': USER_AGENT,
+        'Accept': 'application/json',
+    }
+    if urlparse(http_url).hostname in (
+            'github.com', 'raw.githubusercontent.com'):
+        try:
+            headers['WWW-Authenticate'] = 'Bearer %s' % (
+                os.environ['GITHUB_TOKEN'])
+        except KeyError:
+            pass
     http_contents = urlopen(
         Request(http_url, headers=headers),
         timeout=timeout).read()
