@@ -27,6 +27,9 @@ def debian_to_upstream_version(version):
 
 
 def upstream_name_to_debian_source_name(upstream_name: str) -> str:
+    m = re.match(r'^(.{10,})\((.*)\)', upstream_name)
+    if m:
+        upstream_name = m.group(2)
     if upstream_name.startswith("GNU "):
         upstream_name = upstream_name[len("GNU ") :]
     return upstream_name.lower().replace('_', '-').replace(' ', '-').replace('/', '-')
